/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#pragma once

#include <chrono>
#include <condition_variable>
#include <mutex>
#include <thread>
#include <vector>
#include <string>
#include <functional>

#include <kylin-ai/ai-engine/ondevicevisionengine.h>

class OcrTritonClient;
class MattingTritonClient;
class DdcpTritonClient;
class BinaryImage;

namespace ai_engine::vision {

class OndeviceVisionEngine : public AbstractOnDeviceVisionEngine {
public:

    using OcrResultCallback = std::function<void(TextRecognitionResult result)>;
    using DocumentImageRectificationResultCallback = std::function<void(const ImgBase64Result &result)>;
    using MattingResultCallback = std::function<void(const ImgBase64Result &result)>;
    using BinarizeImageResultCallback = std::function<void(const ImgBase64Result &result)>;
    // bool isCloud() const override;

    OndeviceVisionEngine();
    virtual ~OndeviceVisionEngine();
    
    bool ocrRequestPath(std::string ImagePath, EngineError &error) override;
    bool ocrRequestBase64(const char *ImageData, EngineError &error) override;

    bool DocumentImageRectificationRequestPath(const std::string &ImagePath, EngineError &error) override;
    bool DocumentImageRectificationRequestBase64(const char *ImageData, EngineError &error) override;

    bool mattingRequestPath(const std::string &ImagePath, EngineError &error) override;
    bool mattingRequestBase64(const char *ImageData, EngineError &error) override;
    
    bool binarizeImageRequestPath(const std::string &ImagePath, EngineError &error) override;
    bool binarizeImageRequestBase64(const char *ImageData, EngineError &error) override;
    
    void setOcrCallback(OcrResultCallback callback) override;
    void setDocumentImageRectificationCallback(DocumentImageRectificationResultCallback callback) override;
    void setMattingCallback(MattingResultCallback callback) override;
    void setBinarizeImageCallback(BinarizeImageResultCallback callback) override;

    bool initOcrModule(EngineError &error);
    bool destroyOcrModule(EngineError &error); 

    bool initDocumentImageRectificationModule(EngineError &error);
    bool destroyDocumentImageRectificationModule(EngineError &error); 

    bool initMattingModule(EngineError &error);
    bool destroyMattingModule(EngineError &error); 

    bool initBinarizeImageModule(EngineError &error);
    bool destroyBinarizeImageModule(EngineError &error); 

    std::string engineName() const override { return "ondevice"; };
    std::string currentModelName() const override { return "端侧视觉模型"; };

    std::string modelInfo() const;

private:
    OcrResultCallback ocrResultCallback_ = nullptr;
    DocumentImageRectificationResultCallback ddcpResultCallback_ = nullptr;
    MattingResultCallback mattingResultCallback_ = nullptr;
    BinarizeImageResultCallback imageBinaryzationResultCallback_ = nullptr;

    OcrTritonClient *ocrClient_ = nullptr;
    DdcpTritonClient *ddcpClient_ = nullptr;
    MattingTritonClient *mattingClient_ = nullptr;
    BinaryImage *binImage_ = nullptr;

    std::mutex ocrMutex_;  // 用于保护并发访问的互斥锁
    std::mutex MattingMutex_;  // 用于保护并发访问的互斥锁
    std::mutex DocimgrecMutex_;  // 用于保护并发访问的互斥锁
    std::mutex binImageMutex_;  // 用于保护并发访问的互斥锁
};

class OndeviceDdcpEngine : public AbstractOnDeviceVisionEngine {
public:


    OndeviceDdcpEngine();
    virtual ~OndeviceDdcpEngine();
    
    std::string modelInfo() const;
    


    std::string engineName() const override { return "ondevice"; };
    std::string currentModelName() const override { return "portrait_ddcp"; };

private:
};

class OndeviceMattingEngine : public AbstractOnDeviceVisionEngine {
public:


    OndeviceMattingEngine();
    virtual ~OndeviceMattingEngine();
    
    std::string modelInfo() const;
    


    std::string engineName() const override { return "ondevice"; };
    std::string currentModelName() const override { return "portrait_matting"; };

private:
};
} // namespace ai_engine::vision

